//----------------------------------------------------------------------------
//
// AATEXT - Antialiased text fonts for Allegro
//
// Douglas Eleveld (D.J.Eleveld@anest.azg.nl)
//
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Basic text drawing functions
//----------------------------------------------------------------------------
#include <sys/farptr.h>
#include "internal.h"

#ifdef ALLEGRO_COLOR24

/* Used for fast blending for truecolor aliasing */
extern int _blender_alpha;

/*----------------------------------------------------------------------------
   Very fast 6 level antialiased text blitting for true color modes
   and filled backgrounds.  I don't think it's possible to get
   it much faster than this...
*/
void _antialiased_24bit_filled_render (struct BITMAP *bmp, struct BITMAP *b, int x, int y, int colour)
   {
   int i;

   /* Setup the internal scratchpad abd aliasmap to the correct color depth */
   _prepare_aliasmap(colour,24);

   /* Build each character */
   for(i = 0; i < b->h; i++)
      {
      unsigned char* input = b->line[i];
      unsigned char* output = _scratchpad->line[i];
      const unsigned char* inputend = &b->line[i][b->w];

      /* Super fast blit of a row to temp bitmap */
      while(input!=inputend)
         {
         int c = _aliasmap[*input++>>5];

         *output++ = getr24(c);
         *output++ = getg24(c);
         *output++ = getb24(c);
         }
      }
   /* Blit the character to the destination */
   blit(_scratchpad,bmp,0,0,x,y,b->w,b->h);
   }

/*----------------------------------------------------------------------------
   Pretty fast antialiased text blitting for 24 bit modes and
   empty backgrounds. It fully supports 256 aliasing levels.  Actually
   it works pretty well in all 15,16,24 and 32 bit modes.
*/
void _antialiased_24bit_empty_render (struct BITMAP *bmp, struct BITMAP *b, int x, int y, int colour)
   {
   int i, j;

   set_trans_blender(0, 0, 0, 128);
   drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);

   for(i = y; i < y+b->h; i++)
      {
      unsigned char *input;
      unsigned char *inputend;

      input = b->line[i-y];
      inputend = &b->line[i-y][b->w];

      j = x;

      // Super fast blit of a row to temp bitmap
      while(input!=inputend)
         {
         // Set the translucency from the source character bitmap
         // Plot the text pixel with the correct translucency
         if((_blender_alpha = *input++) > 0)
            putpixel(bmp, j, i, colour);

         j++;
         }
      }
   drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
   }
   
#endif //ALLEGRO_COLOR24

