#define COLOR
#define COUNT
/* store.c, picture output routines                                         */

/* Copyright (C) 1996, MPEG Software Simulation Group. All Rights Reserved. */

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  The MPEG Software Simulation Group disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 * The MPEG Software Simulation Group does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of MPEG-1 and MPEG-2 video, including shareware,
 * are subject to royalty fees to patent holders.  Many of these patents are
 * general enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/movedata.h>

#include "config.h"
#include "global.h"
#include "allegro.h"

/* private prototypes */
static void store_one _ANSI_ARGS_((char *outname, unsigned char *src[],
  int offset, int incr, int height));
static void store_yuv _ANSI_ARGS_((char *outname, unsigned char *src[],
  int offset, int incr, int height));
static void store_sif _ANSI_ARGS_((char *outname, unsigned char *src[],
  int offset, int incr, int height));
static void store_ppm_tga _ANSI_ARGS_((char *outname, unsigned char *src[],
  int offset, int incr, int height, int tgaflag));
static void store_yuv1 _ANSI_ARGS_((char *name, unsigned char *src,
  int offset, int incr, int width, int height));
static void conv422to444 _ANSI_ARGS_((unsigned char *src, unsigned char *dst));
static void conv420to422 _ANSI_ARGS_((unsigned char *src, unsigned char *dst, unsigned char *tempsrc, unsigned char *tempdst));

#define OBFRSIZE 4096
static unsigned char obfr[OBFRSIZE];
static unsigned char *optr;
static int outfile;
extern int XPos;
extern int YPos;

#ifdef COUNT
extern int frame_count;
#endif
static void rot_array (unsigned char *src, unsigned char *dst, int w, int h);

static void rot_array(unsigned char *src, unsigned char *dst, int w, int h)
{
 int i,j;
 unsigned char *p,*dstp;
 for (dstp=dst,i=0;i<h;i++,dstp++)
     for (p=dstp,j=0;j<w;j++,p+=h)
         *p=*src++;
}

/*
 * store a picture as either one frame or two fields
 */
void Write_Frame(src,frame)
unsigned char *src[];
int frame;
//  if (progressive_sequence || progressive_frame || Frame_Store_Flag)
{
  int incr=Coded_Picture_Width,height=vertical_size;
  int i, j;
  int y, u, v, r, g, b;
  int crv, cbu, cgu, cgv;
  unsigned char *py, *pu, *pv;
  static unsigned char *u422, *v422, *u444, *v444, *buf1, *buf2;
  BITMAP *Picture;
//  printf("%d,%d;",incr,height);
#ifdef COUNT
  frame_count++;
#endif
#ifdef COLOR
  if (chroma_format==CHROMA444)
  {
    u444 = src[1];
    v444 = src[2];
  }
  else
  {
    if (!u444)
    {
      if (chroma_format==CHROMA420)
      {
        if (!(u422 = (unsigned char *)malloc((Coded_Picture_Width>>1)
                                             *Coded_Picture_Height)))
          Error("malloc failed");
        if (!(v422 = (unsigned char *)malloc((Coded_Picture_Width>>1)
                                             *Coded_Picture_Height)))
          Error("malloc failed");
        if (!(buf1 = (unsigned char *)malloc((Coded_Picture_Width>>1)
                                             *Coded_Picture_Height)))
          Error("malloc failed");
        if (!(buf2 = (unsigned char *)malloc((Coded_Picture_Width>>1)
                                             *Coded_Picture_Height)))
          Error("malloc failed");
      }

      if (!(u444 = (unsigned char *)malloc(Coded_Picture_Width
                                           *Coded_Picture_Height)))
        Error("malloc failed");

      if (!(v444 = (unsigned char *)malloc(Coded_Picture_Width
                                           *Coded_Picture_Height)))
        Error("malloc failed");
    }

    if (chroma_format==CHROMA420)
    {
      conv420to422(src[1],u422,buf1,buf2);
      conv420to422(src[2],v422,buf1,buf2);
      conv422to444(u422,u444);
      conv422to444(v422,v444);
    }
    else
    {
      conv422to444(src[1],u444);
      conv422to444(src[2],v444);
    }
  }
#endif
  Picture = create_bitmap(horizontal_size, height);
//  clear(Picture);    // I think this line is no use, and cause some snow
#ifdef COLOR
  /* matrix coefficients */
  crv = Inverse_Table_6_9[matrix_coefficients][0];
  cbu = Inverse_Table_6_9[matrix_coefficients][1];
  cgu = Inverse_Table_6_9[matrix_coefficients][2];
  cgv = Inverse_Table_6_9[matrix_coefficients][3];
#endif
//  printf("%d,%d,%d,%d;",crv,cbu,cgu,cgv);
//  py=src[0],pu=u444,pv=v444;
  for (i=0; i<height; i++)
  {
    short temp[horizontal_size];

    py = src[0] + incr*i;
#ifdef COLOR
    pu = u444 + incr*i;
    pv = v444 + incr*i;
#endif

    for (j=0; j<horizontal_size; j++)
    {
#ifdef COLOR
      u = *pu++ - 128;
      v = *pv++ - 128;
#endif
      y = 76309 * (*py++ - 16); /* (255/219)*65536 */
      r = g=b=Clip[y>>16];
#ifdef COLOR
      r = Clip[(y + crv*v + 32768)>>16];
      g = Clip[(y - cgu*u - cgv*v + 32768)>>16];
      b = Clip[(y + cbu*u + 32786)>>16];
#endif
//    putpixel(Picture, j,i,makecol(r,g,b));
      temp[j]=makecol16(r,g,b);
    }
// I think _movedatal() is better. ( movedata() = _movedatab() ? )
//    _movedatal(_my_ds(), (unsigned)temp, Picture->seg, bmp_write_line(Picture, i), horizontal_size>>1);
    movedata(_my_ds(), (unsigned)temp, Picture->seg, bmp_write_line(Picture, i), horizontal_size<<1);
  }
   blit(Picture,screen, 0,0,XPos,YPos, horizontal_size,height);
   destroy_bitmap(Picture);
   free(u422);
   free(v422);
   free(u444);
   free(v444);
   free(buf1);
   free(buf2);
}



/* horizontal 1:2 interpolation filter */
static void conv422to444(src,dst)
unsigned char *src,*dst;
{
  int i, i2, w, j;

  w = Coded_Picture_Width>>1;
  --dst;

  if (base.MPEG2_Flag)
  {
    for (j=0; j<Coded_Picture_Height; j++)
    {
        *(++dst) = src[0];
        *(++dst) = Clip[(int)(128*src[0]+21*src[3]
                        -52*src[2] +159*src[1]+128)>>8];
        *(++dst) = src[1];
        *(++dst) = Clip[(int)(21*src[4]-31*src[0]-52*src[3]
                       +159*(src[1]+src[2])+128)>>8];
      for (i=2; i<w-3; i++)
      {

        /* FIR filter coefficients (*256): 21 0 -52 0 159 256 159 0 -52 0 21 */
        /* even samples (0 0 256 0 0) */
        *(++dst) = src[i];

        /* odd samples (21 -52 159 159 -52 21) */
        *(++dst) = Clip[(int)(21*(src[i-2]+src[i+3])
                        -52*(src[i-1]+src[i+2])
                       +159*(src[i]+src[i+1])+128)>>8];
      }
      *(++dst) = src[w-3];
      *(++dst) = Clip[(int)(21*src[w-5]-52*src[w-4]-31*src[w-1]
                       +159*(src[w-3]+src[w-2])+128)>>8];
      *(++dst) = src[w-2];
      *(++dst) = Clip[(int)(21*src[w-4]+128*src[w-1]
                      -52*src[w-3]+159*src[w-2]+128)>>8];
      *(++dst) = src[w-1];
      *(++dst) = Clip[(int)(21*src[w-3]+287*src[w-1]
                      -52*src[w-2]+128)>>8];
      src+= w;
    }
  }
  else
  {
    for (j=0; j<Coded_Picture_Height; j++)
    {
       *(++dst) =   Clip[(int)(282*src[0]
                         -37*src[1]
                         +11*src[2]+128)>>8];
       *(++dst) = Clip[(int)(  5*src[3]
                         -21*src[2]
                         +70*src[1]
                        +202*src[0]
                         +128)>>8];
       *(++dst) =   Clip[(int)(54*src[0]
                        +228*src[1]
                         -37*src[2]
                         +11*src[3]+128)>>8];

       *(++dst) = Clip[(int)(  5*src[4]
                         -21*src[3]
                         +70*src[2]
                        +228*src[1]
                         -26*src[0]
                         +128)>>8];
       *(++dst) =   Clip[(int)(-16*src[0]
                         +70*src[1]
                        +228*src[2]
                         -37*src[3]
                         +11*src[4]+128)>>8];

       *(++dst) = Clip[(int)(  5*src[5]
                         -21*src[4]
                         +70*src[3]
                        +228*src[2]
                         -37*src[1]
                         +11*src[0]+128)>>8];
      for (i=3; i<w-3; i++)
      {

        /* FIR filter coefficients (*256): 5 -21 70 228 -37 11 */
        *(++dst) =   Clip[(int)(  5*src[i-3]
                         -21*src[i-2]
                         +70*src[i-1]
                        +228*src[i]
                         -37*src[i+1]
                         +11*src[i+2]+128)>>8];

       *(++dst) = Clip[(int)(  5*src[i+3]
                         -21*src[i+2]
                         +70*src[i+1]
                        +228*src[i]
                         -37*src[i-1]
                         +11*src[i-2]+128)>>8];
      }
        *(++dst) =   Clip[(int)(  5*src[w-6]
                         -21*src[w-5]
                         +70*src[w-4]
                        +228*src[w-3]
                         -37*src[w-2]
                         +11*src[w-1]+128)>>8];

       *(++dst) = Clip[(int)(-16*src[w-1]
                         +70*src[w-2]
                        +228*src[w-3]
                         -37*src[w-4]
                         +11*src[w-5]+128)>>8];
        *(++dst) =   Clip[(int)(  5*src[w-5]
                         -21*src[w-4]
                         +70*src[w-3]
                        +228*src[w-2]
                         -26*src[w-1]
                         +128)>>8];

       *(++dst) = Clip[(int)(54*src[w-1]
                        +228*src[w-2]
                         -37*src[w-3]
                         +11*src[w-4]+128)>>8];
        *(++dst) =   Clip[(int)(  5*src[w-4]
                         -21*src[w-3]
                         +70*src[w-2]
                        +202*src[w-1]
                         +128)>>8];

       *(++dst) = Clip[(int)(282*src[w-1]
                         -37*src[w-2]
                         +11*src[w-3]+128)>>8];
      src+= w;
    }
  }
}

/* vertical 1:2 interpolation filter */
static void conv420to422(src,dst,tempsrc,tempdst)
unsigned char *src,*dst,*tempsrc, *tempdst;
{
  int w, h, i, j;
  unsigned char *temp=tempdst;
  w = Coded_Picture_Width>>1;
  h = Coded_Picture_Height>>1;
  rot_array(src,tempsrc,w,Coded_Picture_Height);
  --tempdst;
  if (progressive_frame)
  {
    /* intra frame */
    for (i=0; i<w; i++)
    {
        *(++tempdst) =     Clip[(int)(281*tempsrc[0]
                             -32*tempsrc[1]
                             +7*tempsrc[2]+128)>>8];

        *(++tempdst) = Clip[(int)(  3*tempsrc[3]
                             -16*tempsrc[2]
                             +67*tempsrc[1]
                            +202*tempsrc[0]
                             +128)>>8];
        *(++tempdst) =     Clip[(int)( 54*tempsrc[0]
                            +227*tempsrc[1]
                             -32*tempsrc[2]
                             +7*tempsrc[3]+128)>>8];

        *(++tempdst) = Clip[(int)(  3*tempsrc[4]
                             -16*tempsrc[3]
                             +67*tempsrc[2]
                            +227*tempsrc[1]
                             -25*tempsrc[0]
                             +128)>>8];
        *(++tempdst) =     Clip[(int)(-13*tempsrc[0]
                             +67*tempsrc[1]
                            +227*tempsrc[2]
                             -32*tempsrc[3]
                             +7*tempsrc[4]+128)>>8];

        *(++tempdst) = Clip[(int)(  3*tempsrc[5]
                             -16*tempsrc[4]
                             +67*tempsrc[3]
                            +227*tempsrc[2]
                             -32*tempsrc[1]
                             +7*src[0]+128)>>8];
      for (j=3; j<h-3; j++)
      {

        /* FIR filter coefficients (*256): 5 -21 70 228 -37 11 */
        /* New FIR filter coefficients (*256): 3 -16 67 227 -32 7 */

        *(++tempdst) =     Clip[(int)(  3*tempsrc[(j-3)]
                             -16*tempsrc[(j-2)]
                             +67*tempsrc[(j-1)]
                            +227*tempsrc[j]
                             -32*tempsrc[(j+1)]
                             +7*tempsrc[(j+2)]+128)>>8];

        *(++tempdst) = Clip[(int)(  3*tempsrc[(j+3)]
                             -16*tempsrc[(j+2)]
                             +67*tempsrc[(j+1)]
                            +227*tempsrc[j]
                             -32*tempsrc[(j-1)]
                             +7*tempsrc[(j-2)]+128)>>8];
      }
        *(++tempdst) = Clip[(int)(  3*tempsrc[(h-6)]
                             -16*tempsrc[(h-5)]
                             +67*tempsrc[(h-4)]
                            +227*tempsrc[(h-3)]
                             -32*tempsrc[(h-2)]
                             +7*tempsrc[(h-1)]+128)>>8];

        *(++tempdst) = Clip[(int)(-13*tempsrc[(h-1)]
                             +67*tempsrc[(h-2)]
                            +227*tempsrc[(h-3)]
                             -32*tempsrc[(h-4)]
                             +7*tempsrc[(h-5)]+128)>>8];
        *(++tempdst) = Clip[(int)(  3*tempsrc[(h-5)]
                             -16*tempsrc[(h-4)]
                             +67*tempsrc[(h-3)]
                            +227*tempsrc[(h-2)]
                             -25*tempsrc[(h-1)]
                             +128)>>8];

        *(++tempdst) = Clip[(int)( 54*tempsrc[(h-1)]
                            +227*tempsrc[(h-2)]
                             -32*tempsrc[(h-3)]
                             +7*tempsrc[(h-4)]+128)>>8];
        *(++tempdst) = Clip[(int)(  3*tempsrc[(h-4)]
                             -16*tempsrc[(h-3)]
                             +67*tempsrc[(h-2)]
                            +202*tempsrc[(h-1)]
                             +128)>>8];

        *(++tempdst) = Clip[(int)( 281*tempsrc[(h-1)]
                             -32*tempsrc[(h-2)]
                             +7*tempsrc[(h-3)]+128)>>8];
      tempsrc+=Coded_Picture_Height;
    }
  }
  else
  {
    /* intra field */
    for (i=0; i<w; i++)
    {
        *(++tempdst) = Clip[(int)( 272*tempsrc[0]
                         -21*tempsrc[2]
                          +5*tempsrc[4]+128)>>8];

        *(++tempdst) = Clip[(int)( 7*tempsrc[5]
                             -35*tempsrc[3]
                            +284*tempsrc[1]
                            +128)>>8];

        *(++tempdst) = Clip[(int)(166*tempsrc[0]
                            +110*tempsrc[2]
                             -24*tempsrc[4]
                              +4*tempsrc[6]+128)>>8];


        *(++tempdst) = Clip[(int)(  1*tempsrc[7]
                             -7*tempsrc[5]
                             +30*tempsrc[3]
                            +232*tempsrc[1]
                              +128)>>8];
        *(++tempdst) = Clip[(int)( 24*tempsrc[0]
                        +248*tempsrc[2]
                         -21*tempsrc[4]
                          +5*tempsrc[6]+128)>>8];

        *(++tempdst) = Clip[(int)( 7*tempsrc[7]
                             -35*tempsrc[5]
                            +194*tempsrc[3]
                            +90*tempsrc[1]
                            +128)>>8];

        *(++tempdst) = Clip[(int)(-28*tempsrc[0]
                            +194*tempsrc[2]
                            +110*tempsrc[4]
                             -24*tempsrc[6]
                              +4*tempsrc[8]+128)>>8];

        *(++tempdst) = Clip[(int)(  1*tempsrc[9]
                             -7*tempsrc[7]
                             +30*tempsrc[5]
                            +248*tempsrc[3]
                             -16*tempsrc[1]
                             +128)>>8];
        *(++tempdst) = Clip[(int)( -6*tempsrc[0]
                         +30*tempsrc[2]
                        +248*tempsrc[4]
                         -21*tempsrc[6]
                          +5*tempsrc[8]+128)>>8];

        *(++tempdst) = Clip[(int)( 7*tempsrc[9]
                             -35*tempsrc[7]
                            +194*tempsrc[5]
                            +110*tempsrc[3]
                             -20*tempsrc[1]
                              +128)>>8];

        *(++tempdst) = Clip[(int)( 7*tempsrc[0]
                             -35*tempsrc[2]
                            +194*tempsrc[4]
                            +110*tempsrc[6]
                             -24*tempsrc[8]
                              +4*tempsrc[10]+128)>>8];

        *(++tempdst) = Clip[(int)(  1*tempsrc[11]
                             -7*tempsrc[9]
                             +30*tempsrc[7]
                            +248*tempsrc[5]
                             -21*tempsrc[3]
                              +5*tempsrc[1]+128)>>8];
     for (j=6; j<h-6; j+=2)
      {
        /* top field */

        /* Polyphase FIR filter coefficients (*256): 2 -10 35 242 -18 5 */
        /* New polyphase FIR filter coefficients (*256): 1 -7 30 248 -21 5 */
        *(++tempdst) = Clip[(int)(  1*tempsrc[(j-6)]
                         -7*tempsrc[(j-4)]
                         +30*tempsrc[(j-2)]
                        +248*tempsrc[j]
                         -21*tempsrc[(j+2)]
                          +5*tempsrc[(j+4)]+128)>>8];

        *(++tempdst) = Clip[(int)( 7*tempsrc[(j+5)]
                             -35*tempsrc[(j+3)]
                            +194*tempsrc[(j+1)]
                            +110*tempsrc[(j-1)]
                             -24*tempsrc[(j-3)]
                              +4*tempsrc[(j-5)]+128)>>8];


        /* Polyphase FIR filter coefficients (*256): 11 -38 192 113 -30 8 */
        /* New polyphase FIR filter coefficients (*256):7 -35 194 110 -24 4 */
        *(++tempdst) = Clip[(int)( 7*tempsrc[(j-4)]
                             -35*tempsrc[(j-2)]
                            +194*tempsrc[j]
                            +110*tempsrc[(j+2)]
                             -24*tempsrc[(j+4)]
                              +4*tempsrc[(j+6)]+128)>>8];

        /* bottom field */

        /* Polyphase FIR filter coefficients (*256): 11 -38 192 113 -30 8 */
        /* New polyphase FIR filter coefficients (*256):7 -35 194 110 -24 4 */
        *(++tempdst) = Clip[(int)(  1*tempsrc[(j+7)]
                             -7*tempsrc[(j+5)]
                             +30*tempsrc[(j+3)]
                            +248*tempsrc[(j+1)]
                             -21*tempsrc[(j-1)]
                              +5*tempsrc[(j-3)]+128)>>8];
      }
        *(++tempdst) = Clip[(int)(  1*tempsrc[(h-12)]
                         -7*tempsrc[(h-10)]
                         +30*tempsrc[(h-8)]
                        +248*tempsrc[(h-6)]
                         -21*tempsrc[(h-4)]
                          +5*tempsrc[(h-2)]+128)>>8];
        *(++tempdst) = Clip[(int)( 7*tempsrc[(h-1)]
                             -35*tempsrc[(h-3)]
                            +194*tempsrc[(h-5)]
                            +110*tempsrc[(h-7)]
                             -24*tempsrc[(h-9)]
                              +4*tempsrc[(h-11)]+128)>>8];
        *(++tempdst) = Clip[(int)( 7*tempsrc[(h-10)]
                             -35*tempsrc[(h-8)]
                            +194*tempsrc[(h-6)]
                            +110*tempsrc[(h-4)]
                             -20*tempsrc[(h-2)]
                             +128)>>8];
        *(++tempdst) = Clip[(int)( -6*tempsrc[(h-1)]
                             +30*tempsrc[(h-3)]
                            +248*tempsrc[(h-5)]
                             -21*tempsrc[(h-7)]
                              +5*tempsrc[(h-9)]+128)>>8];
        *(++tempdst) = Clip[(int)(  1*tempsrc[(h-10)]
                         -7*tempsrc[(h-8)]
                         +30*tempsrc[(h-6)]
                        +248*tempsrc[(h-4)]
                         -16*tempsrc[(h-2)]
                          +128)>>8];
        *(++tempdst) = Clip[(int)( -28*tempsrc[(h-1)]
                            +194*tempsrc[(h-3)]
                            +110*tempsrc[(h-5)]
                             -24*tempsrc[(h-7)]
                              +4*tempsrc[(h-9)]+128)>>8];
        *(++tempdst) = Clip[(int)( 7*tempsrc[(h-8)]
                             -35*tempsrc[(h-6)]
                            +194*tempsrc[(h-4)]
                            +90*tempsrc[(h-2)]
                             +128)>>8];

        *(++tempdst) = Clip[(int)( 24*tempsrc[(h-1)]
                            +248*tempsrc[(h-3)]
                             -21*tempsrc[(h-5)]
                              +5*tempsrc[(h-7)]+128)>>8];
        *(++tempdst) = Clip[(int)(  1*tempsrc[(h-8)]
                         -7*tempsrc[(h-6)]
                         +30*tempsrc[(h-4)]
                        +232*tempsrc[(h-2)]
                          +128)>>8];
        *(++tempdst) = Clip[(int)(166*tempsrc[(h-1)]
                            +110*tempsrc[(h-3)]
                             -24*tempsrc[(h-5)]
                              +4*tempsrc[(h-7)]+128)>>8];
        *(++tempdst) = Clip[(int)( 7*tempsrc[(h-6)]
                             -35*tempsrc[(h-4)]
                            +284*tempsrc[j]
                              +128)>>8];

        *(++tempdst) = Clip[(int)(272*tempsrc[(h-1)]
                             -21*tempsrc[(h-3)]
                              +5*tempsrc[(h-5)]+128)>>8];
      tempsrc+=Coded_Picture_Height;
    }
  }
  rot_array(temp,dst,Coded_Picture_Height,w);

}
